# Copyright (c) 2016 The aionotify project
# This code is distributed under the two-clause BSD License.

from .enums import Flags
from .base import Watcher

__all__ = ['Flags', 'Watcher']

__version__ = '0.2.0'
__author__ = 'Raphaël Barrois <raphael.barrois+aionotify@polytechnique.org>'
